// [kwrited.C] A write(1) receiver for kde.

#ifdef FD_SETSIZE
#undef FD_SETSIZE
#endif
#define FD_SETSIZE 8192

#include <dcopclient.h>
#include <qsocketnotifier.h>

#include <kuniqueapplication.h>
#include <kcmdlineargs.h>
#include <klocale.h>
#include <kglobalsettings.h>
#include <kdebug.h>
#include <kcrash.h>
#include <kpty.h>
#include <kuser.h>
#include <kglobal.h>

#include "kwrited.h"

#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <signal.h>
#include <strings.h>
#include <sys/types.h>
#include <sys/resource.h>
#include <errno.h>
#include <X11/Xlib.h>
#include <fixx11h.h>

#include <config.h>

extern "C" {
    static void setSystemLimits (void)
    {
        static struct rlimit maxLimit;
        (void) memset (&maxLimit, '\0', sizeof (struct rlimit));

	if (getrlimit (RLIMIT_NOFILE, &maxLimit) < 0)
	{
	    perror ("getrlimit"), strerror (errno);
	    return;
	}

	if (maxLimit.rlim_cur < 1024)
	    maxLimit.rlim_cur = 1024;
	if (maxLimit.rlim_max < 8192)
	    maxLimit.rlim_max = 8192;

        if (setrlimit (RLIMIT_NOFILE, &maxLimit) < 0)
            perror ("setrlimit"), strerror (errno);
        return;
    }         
}
  
extern "C"
{
    typedef void SignalHandlerFunction (int);
    static SignalHandlerFunction* KSignal (int signo, SignalHandlerFunction* sigFunc)
    {
        static struct sigaction act;
	static struct sigaction oact;
        act.sa_handler = sigFunc;
        sigemptyset (&act.sa_mask);
        act.sa_flags = 0;
        if (signo == SIGALRM)
        {
#ifdef SA_INTERRUPT
            act.sa_flags |= SA_INTERRUPT;
#endif
        }
        else
        {
            act.sa_flags |= SA_RESTART;
        }
        if (sigaction (signo, &act, &oact) != 0)
                return (SIG_ERR);
        return (oact.sa_handler);
    }
}

extern "C"
{
    static void KSignalHandler (int signo)
    {
	KSignal (signo, SIG_IGN);
	KSignal (signo, KSignalHandler);
    }
}


/* TODO
   for anyone who likes to do improvements here, go ahead.
   - check FIXMEs below
   - add Menu
     - accept messages (on/off)
     - pop up on incoming messages
     - clear messages
     - allow max. lines
   - add CORBA interface?
   - add session awareness.
   - add client complements.
   - kwrited is disabled by default if built without utempter,
     see ../Makefile.am - kwrited doesn't seem to work well without utempter
*/

KWrited::KWrited()
  : QObject(),
  wid(0L),
  pty(0L)
{
  KSignal(SIGPIPE, KSignalHandler);
  KSignal(SIGALRM, KSignalHandler);

  wid = new QTextEdit(0, "messages");
  wid->setFont(KGlobalSettings::fixedFont());
  wid->setMinimumWidth(wid->fontMetrics().maxWidth()*80 +
      wid->minimumSizeHint().width());
  wid->setReadOnly(true);
  wid->setFocusPolicy(QWidget::NoFocus);

  pty = new KPty();
  pty->open();
  pty->login(KUser().loginName().local8Bit().data(), getenv("DISPLAY"));
  QSocketNotifier *sn = new QSocketNotifier(pty->masterFd(), QSocketNotifier::Read, this);
  connect(sn, SIGNAL(activated(int)), this, SLOT(block_in(int)));

  QString txt = i18n("KWrited - Listening on Device %1").arg(pty->ttyName());
  wid->setCaption(txt);
  puts(txt.local8Bit().data());
}

KWrited::KWrited(const KWrited& rhs)
  : QObject(),
  wid(rhs.wid),
  pty(rhs.pty)
{
}


KWrited::~KWrited()
{
    pty->logout();
    delete pty;
    pty = 0L;
}

KWrited&
KWrited::operator= (const KWrited& rhs)
{
  if (this != &rhs)
  {
     wid = rhs.wid;
     pty = rhs.pty;
  }
  return *this;
}

void KWrited::block_in(int fd)
{
  char buf[4096];
  int len = read(fd, buf, 4096);
  if (len <= 0)
     return;

  wid->insert( QString::fromLocal8Bit( buf, len ).remove('\r') );
  wid->show();
  wid->raise();
}

KWritedModule::KWritedModule( const QCString& obj )
    : KDEDModule( obj ),
    pro(0L)
{
    KGlobal::locale()->insertCatalogue("konsole");
    pro = new KWrited();
}

KWritedModule::KWritedModule(const KWritedModule& rhs)
  : KDEDModule(QCString(rhs.name())),
  pro(rhs.pro)
{
}

KWritedModule::~KWritedModule()
{
    delete pro;
    pro = 0L;
    KGlobal::locale()->removeCatalogue("konsole");
}

KWritedModule&
KWritedModule::operator= (const KWritedModule& rhs)
{
    if (this != &rhs)
    {
	pro = rhs.pro;
    }
    return *this;
}

extern "C"
KDE_EXPORT KDEDModule* create_kwrited( const QCString& obj )
{
    return new KWritedModule( obj );
}

#include "kwrited.moc"
