/**************************************************************************

    main.cpp  - The main function for KPager
    Copyright (C) 1998-2000  Antonio Larrosa Jimenez

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

    Send comments and bug fixes to larrosa@kde.org

***************************************************************************/

#ifdef FD_SETSIZE
#undef FD_SETSIZE
#endif
#define FD_SETSIZE 8192

#include <kuniqueapplication.h>
#include <klocale.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <qcolor.h>
#include <kdebug.h>
#include <stdlib.h>
#include <qsessionmanager.h>

#include "kpager.h"

#include <strings.h>
#include <signal.h>
#include <sys/types.h>
#include <sys/resource.h>
#include <errno.h>

extern "C" {
    static void setSystemLimits (void)
    {
        static struct rlimit maxLimit;
        (void) memset (&maxLimit, '\0', sizeof (struct rlimit));

	if (getrlimit (RLIMIT_NOFILE, &maxLimit) < 0)
	{
	    perror ("getrlimit"), strerror (errno);
	    return;
	}

	if (maxLimit.rlim_cur < 1024)
	    maxLimit.rlim_cur = 1024;
	if (maxLimit.rlim_max < 8192)
	    maxLimit.rlim_max = 8192;

        if (setrlimit (RLIMIT_NOFILE, &maxLimit) < 0)
            perror ("setrlimit"), strerror (errno);
        return;
    }         
}
  
extern "C"
{
    typedef void SignalHandlerFunction (int);
    static SignalHandlerFunction* KSignal (int signo, SignalHandlerFunction* sigFunc)
    {
        static struct sigaction act;
	static struct sigaction oact;
        act.sa_handler = sigFunc;
        sigemptyset (&act.sa_mask);
        act.sa_flags = 0;
        if (signo == SIGALRM)
        {
#ifdef SA_INTERRUPT
            act.sa_flags |= SA_INTERRUPT;
#endif
        }
        else
        {
            act.sa_flags |= SA_RESTART;
        }
        if (sigaction (signo, &act, &oact) != 0)
                return (SIG_ERR);
        return (oact.sa_handler);
    }
}

extern "C"
{
    static void KSignalHandler (int signo)
    {
	KSignal (signo, SIG_IGN);
	KSignal (signo, KSignalHandler);
    }
}


static KCmdLineOptions pagerOpts[] =
{
    { "hidden", I18N_NOOP("Create pager but keep the window hidden"), 0 },
    KCmdLineLastOption
};

bool closed_by_sm = false;

class KPagerApplication : public KUniqueApplication
{
public:
  KPagerApplication() : KUniqueApplication() {}

  void commitData(QSessionManager& sm) {
    if (mainWidget()->isHidden()) {
      sm.setRestartHint( QSessionManager::RestartNever );
      return;
    }
    closed_by_sm = true;
    KUniqueApplication::commitData( sm );
    closed_by_sm = false;
  }

  int newInstance() {
    mainWidget()->show();
    return 0;
  }

};

int main(int argc, char **argv)
{
    setSystemLimits();
    KSignal (SIGPIPE, KSignalHandler);
    KSignal (SIGALRM, KSignalHandler);
    KSignal (SIGHUP, KSignalHandler);
    KSignal (SIGTERM, KSignalHandler);

    KAboutData *aboutdata = new KAboutData("kpager", "KPager", "1.5",
					   I18N_NOOP("Desktop Overview"), KAboutData::License_GPL,
					   "(C) 1998-2002, Antonio Larrosa Jimenez","",
					   "http://developer.kde.org/~larrosa/kpager.html");

    aboutdata->addAuthor("Antonio Larrosa Jimenez",
			 I18N_NOOP("Original Developer/Maintainer"),"larrosa@kde.org",
			 "http://developer.kde.org/~larrosa/index.html");
    aboutdata->addAuthor("Matthias Elter",
			 I18N_NOOP("Developer"),"elter@kde.org", "");
    aboutdata->addAuthor("Matthias Ettrich",
			 I18N_NOOP("Developer"),"ettrich@kde.org", "");

    KCmdLineArgs::init(argc, argv, aboutdata);
    KCmdLineArgs::addCmdLineOptions(pagerOpts);
    KUniqueApplication::addCmdLineOptions();

    if (!KUniqueApplication::start())
    {
      kdError() << "kpager is already running!" << endl;
      return 0;
    }


    KApplication * app = new KPagerApplication;
    KCmdLineArgs *args = KCmdLineArgs::parsedArgs();

    KPagerMainWindow *kpager = new KPagerMainWindow(0,"KPager");
    kpager->setPlainCaption( i18n("Desktop Pager") );


    app->setMainWidget(kpager);
    if (!args->isSet("hidden")) kpager->show();
    else kpager->hide();

    int ret = app->exec();

    delete app;
    return ret;
}

